<?php
/**
 * Plugin Name: SW-Retail for WooCommerce
 * Plugin URI: https://www.mull2media.nl
 * Description: This plugin lets synchronize SW-Retail products to WooCommerce and send orders from Woocommerce to SW-Retail
 * Version: 3.10.1
 * Author: Mull2media
 * Author URI: https://www.mull2media.nl/
 * Text Domain: swretail
 */


require_once(dirname(__FILE__).'/plugin-update-checker/plugin-update-checker.php');
$MyUpdateChecker = PucFactory::buildUpdateChecker(
    'https://swretail.mull2media.nl/woocommerce-sw-retail.php',
//    'https://swretail.mull2media.nl/woocommerce-sw-retail.json',
    __FILE__,
    'swretail'
);

function swretailShowVersion(){
    $json = file_get_contents('https://swretail.mull2media.nl/woocommerce-sw-retail.php');
    $obj = json_decode($json);
    return $obj->version;
}
add_shortcode('swretail_show_version','swretailShowVersion');


//Requires WooCommerce
if(in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))){
    require_once(dirname(__FILE__).'/apiservice.php');

    global $swretail_db_version;
    $swretail_db_version = '1.0';
    $swretail_options = get_option('swretail_options',array());
    $swretail_mapping = get_option('swretail_mapping',array());
    $swretail_addons = get_option('swretail_addons',array());
    $swretail_meta = get_option('swretail_meta',array());
    $swretail_combine_attributes = get_option('swretail_combine_attributes',array());

    function swretail_init(){
        load_plugin_textdomain('swretail', false, dirname( plugin_basename( __FILE__ )));
    }
    add_action('init', 'swretail_init');

    if(isset($swretail_addons['external_stock']['active']) && $swretail_addons['external_stock']['active'] == 1){
        require_once(dirname(__FILE__).'/addons/external-stock-addon.php');
    }

    function swretail_cron_schedules($schedules){
        if(!isset($schedules["4weeks"])){
            $schedules["4weeks"] = array(
                'interval' => 60*60*24*7*4,
                'display' => __('Once every 4 weeks'));
        }
        if(!isset($schedules["24hour"])){
            $schedules["24hour"] = array(
                'interval' => 24*60*60,
                'display' => __('Once every 24 hour'));
        }
        if(!isset($schedules["1hour"])){
            $schedules["1hour"] = array(
                'interval' => 60*60,
                'display' => __('Once every hour'));
        }
        if(!isset($schedules["30min"])){
            $schedules["30min"] = array(
                'interval' => 30*60,
                'display' => __('Once every 30 minutes'));
        }
        if(!isset($schedules["5min"])){
            $schedules["5min"] = array(
                'interval' => 5*60,
                'display' => __('Once every 5 minutes'));
        }
        if(!isset($schedules["1min"])){
            $schedules["1min"] = array(
                'interval' => 1*60,
                'display' => __('Once every minutes'));
        }
        return $schedules;
    }
    add_filter('cron_schedules', 'swretail_cron_schedules');
    register_activation_hook(__FILE__, 'swretail_install');

    function swretail_install(){
        global $wpdb, $swretail_db_version;
        $charset_collate = $wpdb->get_charset_collate();

        $swretail_log_table = $wpdb->prefix . "swretail_log";
        $swretail_log_sql = "CREATE TABLE $swretail_log_table (
            ID int(11) NOT NULL AUTO_INCREMENT,
            time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
            text text NOT NULL,
            PRIMARY KEY (id)
        ) $charset_collate;";

        $licence = get_option('swretail_licence');
        if(empty($licence)){
            update_option('swretail_licence', time());
        }

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $swretail_log_sql );

        add_option( 'swretail_db_version', $swretail_db_version );

        //Tasks
        if(!wp_next_scheduled('swretail_get_changes')){
            wp_schedule_event(time(), '24hour', 'swretail_event_check_license');
            wp_schedule_event(time(), '30min', 'swretail_event_get_product_changes');
            wp_schedule_event(time(), '5min', 'swretail_event_get_order_changes');
            wp_schedule_event(time(), '5min', 'swretail_event_get_stock_changes');
            wp_schedule_event(time(), '4weeks', 'swretail_event_check_wc_products');
        }
    }

    add_action('wp', 'swretail_cronstarter');
    function swretail_cronstarter(){
        global $swretail_options,$swretail_addons;
        if(!wp_next_scheduled('swretail_event_check_license')){
            wp_schedule_event(time(), '24hour', 'swretail_event_check_license');
        }
        if(!wp_next_scheduled('swretail_event_get_product_changes')){
            if($swretail_options['sync'] != 'stock_only'){
                wp_schedule_event(time(), '30min', 'swretail_event_get_product_changes');
            }
        }
        if(!wp_next_scheduled('swretail_event_get_order_changes')){
            wp_schedule_event(time(), '5min', 'swretail_event_get_order_changes');
        }
        if(!wp_next_scheduled('swretail_event_get_stock_changes')){
            if($swretail_options['stock_sync_freq'] == '1min_office_hours'){
                wp_schedule_event(time(), '1min', 'swretail_event_get_stock_changes');
            }elseif($swretail_options['stock_sync_freq'] == '30min'){
                wp_schedule_event(time(), '30min', 'swretail_event_get_stock_changes');
            }else{
                wp_schedule_event(time(), '5min', 'swretail_event_get_stock_changes');
            }
        }
        
        if(isset($swretail_addons['wwpp_wholesale']['active']) && $swretail_addons['wwpp_wholesale']['active'] == 1){
            if(!wp_next_scheduled('swretail_event_wholesale_reinitialize')){
                wp_schedule_event(time(), '1hour', 'swretail_event_wholesale_reinitialize');
            }
        }

        if(isset($swretail_addons['external_stock']['active']) && $swretail_addons['external_stock']['active'] == 1){
            if(!wp_next_scheduled('swretail_event_update_external_stock')){ 

                // DEBUGGING
                // wp_schedule_event(time(), '5min', 'swretail_event_update_external_stock');
                
                $update_time = '03:00';
                if(isset($swretail_addons['external_stock']['update_time']) && !empty($swretail_addons['external_stock']['update_time'])){
                    $update_time = $swretail_addons['external_stock']['update_time'];
                }
 
                $next_event = strtotime($update_time);
                if(strtotime($update_time) < current_time('timestamp')){
                    $next_event = strtotime('tomorrow '.$update_time); 
                }
                wp_schedule_single_event($next_event,'swretail_event_update_external_stock');
            }
        }
        
        
        if(!wp_next_scheduled('swretail_event_check_wc_products')){
            if(isset($swretail_options['delete_products']) && $swretail_options['delete_products'] == 'no'){return;}
            wp_schedule_event(time(), '4weeks', 'swretail_event_check_wc_products');
        }
    }

    register_deactivation_hook(__FILE__, 'swretail_uninstall');
    function swretail_uninstall() {
        wp_clear_scheduled_hook('swretail_event_get_product_changes');
        wp_clear_scheduled_hook('swretail_event_get_stock_changes');
        wp_clear_scheduled_hook('swretail_event_get_order_changes');
        wp_clear_scheduled_hook('swretail_event_check_license');
        wp_clear_scheduled_hook('swretail_event_check_wc_products');
        wp_clear_scheduled_hook('swretail_event_wholesale_reinitialize');
        wp_clear_scheduled_hook('swretail_event_update_external_stock');
    }

    add_action('woocommerce_order_status_on-hold', 'swretail_insert_order');
    //add_action('woocommerce_order_status_processing', 'swretail_insert_order');
    add_action('woocommerce_order_status_completed', 'swretail_insert_order');
    add_action('woocommerce_payment_complete', 'swretail_insert_order');
    function swretail_insert_order($orderId){
        if(!empty($orderId)){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
            if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
                $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
                $swRetail->postSwOrder(new WC_Order($orderId));
            }
        }
    }

    add_action('swretail_event_process_changes', 'swretail_process_scheduled_changes', 10, 2);
    function swretail_process_scheduled_changes($productsIDs = '', $stockOnly = false){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getSwProductsByIds($productsIDs, $stockOnly);
        }
    }

    add_action('swretail_event_process_orders', 'swretail_process_scheduled_orders', 10, 2);
    function swretail_process_scheduled_orders($order, $attempt = 1){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->postSwOrder(new WC_Order($order), $attempt);
        }
    }

    add_action('swretail_event_get_product_changes', 'swretail_get_product_changes');
    function swretail_get_product_changes(){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getChangedSwProducts();
        }
    }
    add_action('swretail_event_get_order_changes', 'swretail_get_order_changes');
    function swretail_get_order_changes(){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getChangedSwOrders();
        }
    }

    add_action('swretail_event_get_stock_changes', 'swretail_get_stock_changes');
    function swretail_get_stock_changes(){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
        
        $minutes = 10; 
        if($swretail_options['stock_sync_freq'] == '1min_office_hours'){
            $minutes = 5; 
            if(current_time('G') < 10 || current_time('G') > 18){return false;}
        }
        if($swretail_options['stock_sync_freq'] == '30min'){
            $minutes = 30;
        }
        
        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getChangedSwStock($minutes);
        } 
        update_option('swretail_last_cronjob',current_time('timestamp'));
    }

    add_action( 'wp_ajax_swretail_get_all_products', 'swretail_get_all_products' );
    function swretail_get_all_products() {
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;

        if(isset($_POST['page']) && $_POST['page'] == 0){
            update_option('swretail_categories',array());
        }
        $categories_only = false;
        if(isset($_POST['categoriesOnly']) && $_POST['categoriesOnly'] == 'true'){
            $categories_only = true;
        }

        $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
        $result = $swRetail->getAllSwProducts($_POST['page'],$categories_only);

        echo json_encode($result);

        wp_die(); // this is required to terminate immediately and return a proper response
    }

    add_action('swretail_event_check_license', 'swretail_check_license');
    function swretail_check_license(){
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;
        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->checkLicence();
        }
    }

    add_action('swretail_event_check_wc_products', 'swretail_check_wc_products');
    function swretail_check_wc_products(){
        $swretail_options = get_option('swretail_options');
        if(isset($swretail_addons['syncbybarcodes']['active']) && $swretail_addons['syncbybarcodes']['active'] == 1){return;}
        if(isset($swretail_options['delete_products']) && $swretail_options['delete_products'] == 'no'){return;}

        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getAllWcProducts();
        }
    }

    add_action('swretail_event_wholesale_reinitialize', 'swretail_wholesale_reinitialize');
    function swretail_wholesale_reinitialize(){
        if( class_exists( 'WooCommerceWholeSalePricesPremium' ) ){
            $wwpp = WooCommerceWholeSalePricesPremium::instance();
            $wwpp_wholesale_roles = WWPP_Wholesale_Roles::instance();
            $wwpp_wholesale_role_payment_gateway = WWPP_Wholesale_Role_Payment_Gateway::instance( array( 'WWPP_Wholesale_Roles' => $wwpp_wholesale_roles ) );
            $wwpp_bootstrap = WWPP_Bootstrap::instance( array(
                'WWPP_Wholesale_Roles' => $wwpp_wholesale_roles,
                'WWPP_Wholesale_Role_Payment_Gateway' => $wwpp_wholesale_role_payment_gateway,
                'WWPP_CURRENT_VERSION' => $wwpp::VERSION
            ) );
            $wwpp_bootstrap->initialize_product_visibility_filter_meta();
        }
    }

    /**
     * Add a custom action to order actions select box on edit order page
     *
     * @param array $actions order actions array to display
     * @return array - updated actions
     */
    function swretail_wc_add_export_order_meta_box_action( $actions ) {
        global $theorder;

        if (get_post_meta( $theorder->get_id(), '_wc_order_send_to_swretail', true ) ) {
            $actions['wc_order_to_swretail_action'] = __( 'Resend to SW-Retail', 'swretail' );
        }else{
            $actions['wc_order_to_swretail_action'] = __( 'Send to SW-Retail', 'swretail' );
        }

        return $actions;
    }
    add_action( 'woocommerce_order_actions', 'swretail_wc_add_export_order_meta_box_action' );

    function swretail_wc_process_order_send_action( $order ) {
        $swretail_options = get_option('swretail_options');
        //global $swretail_options;

        if(!empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->postSwOrder($order);
        }
    }
    add_action( 'woocommerce_order_action_wc_order_to_swretail_action', 'swretail_wc_process_order_send_action' );


    /**
     * Add a extra column to the order table to see if a order is in SW-retail.
     *
     * @param $columns
     * @return array
     */
    function wc_sw_order_column( $columns ) {

        $new_columns = array();
        foreach ( $columns as $column_name => $column_info ) {

            $new_columns[ $column_name ] = $column_info;

            if ( 'order_total' === $column_name ) {
                $new_columns['swretail'] = 'SW-retail';
            }
        }
        return $new_columns;
    }
    add_filter( 'manage_edit-shop_order_columns', 'wc_sw_order_column' );


    /**
     * Adds 'SW-retail' column content to 'Orders' page immediately after 'Total' column.
     *
     * @param string[] $column name of column being displayed
     */
    function sv_wc_cogs_add_order_profit_column_content( $column ) {
        global $post;

        if ( 'swretail' === $column ) {
            if ($exported = get_post_meta($post->ID, '_wc_order_send_to_swretail', true) ) {
                echo '<div title="'.$exported.'" style="height:10px; width:10px; background:#52c14a; -webkit-border-radius:5px; -moz-border-radius:5px; border-radius:5px;"></div>';
            }else{
                echo '<div style="height:10px; width:10px; background:#C13531; -webkit-border-radius:5px; -moz-border-radius:5px; border-radius:5px;"></div>';
            }
        }
    }
    add_action( 'manage_shop_order_posts_custom_column', 'sv_wc_cogs_add_order_profit_column_content' );

    add_action('admin_notices', 'swretail_synchronization_error_notices');
    function swretail_synchronization_error_notices() {
        $swretail_last_error = get_option('swretail_last_error');

        $plugin_data = get_plugin_data( dirname(__FILE__).'/swretail.php' );
        $plugin_version = $plugin_data['Version'];
        $filename = 'log-'.str_replace('.','-',$plugin_version);

        if(file_exists(dirname(__FILE__).'/log/'.$filename.'.txt')){
            $log = file_get_contents(dirname(__FILE__).'/log/'.$filename.'.txt');
            if(!empty($swretail_last_error) && $swretail_last_error > strtotime("-3 days") && !empty($log)){
                echo '<div class="notice notice-error">
                    <p>'.__( 'The SW-retail plugin experienced some problems lately, please check the log on the SW-retail page for more details.', 'swretail' ).'</p>
                </div>';
            }
        }
    }

    function swretail_auto_load(){
        global $swretail_options, $swretail_mapping, $swretail_addons, $swretail_meta, $swretail_combine_attributes;
        swretail_get_order_changes();
        if(!empty($_POST['swretail_options'])){
            
            if(!empty($_POST['swretail_options']['licence_key'])){
                $swretail_options['licence_key'] = $_POST['swretail_options']['licence_key'];
                $swretail_options['api_url'] = $_POST['swretail_options']['api_url'];
                $swretail_options['api_user'] = $_POST['swretail_options']['api_user'];
            }

            if(!empty($_POST['swretail_options']['tax_rate_default'])){
                $swretail_options['tax_rate_default'] = $_POST['swretail_options']['tax_rate_default'];
            }
            if(!empty($_POST['swretail_options']['tax_rate_reduced'])){
                $swretail_options['tax_rate_reduced'] = $_POST['swretail_options']['tax_rate_reduced'];
            }
            if(!empty($_POST['swretail_options']['backorders'])){
                $swretail_options['backorders'] = $_POST['swretail_options']['backorders'];
            }
            if(!empty($_POST['swretail_options']['webprice'])){
                $swretail_options['webprice'] = $_POST['swretail_options']['webprice'];
            }else{
                $swretail_options['webprice'] = '';
            }
            if(!empty($_POST['swretail_options']['credits'])){
                $swretail_options['credits'] = $_POST['swretail_options']['credits'];
            }
            if(!empty($_POST['swretail_options']['sku_field'])){
                $swretail_options['sku_field'] = $_POST['swretail_options']['sku_field'];
            }
            if(!empty($_POST['swretail_options']['sync'])){
                $swretail_options['sync'] = $_POST['swretail_options']['sync'];
                wp_clear_scheduled_hook('swretail_event_get_product_changes');
            }
            if(!empty($_POST['swretail_options']['stock_sync_freq'])){
                $swretail_options['stock_sync_freq'] = $_POST['swretail_options']['stock_sync_freq'];
                // Clear stock change cronjob on setting-save
                wp_clear_scheduled_hook('swretail_event_get_stock_changes');

                if($swretail_options['stock_sync_freq'] == '1min_office_hours'){
                    wp_schedule_event(time(), '1min', 'swretail_event_get_stock_changes');
                }elseif($swretail_options['stock_sync_freq'] == '30min'){
                    wp_schedule_event(time(), '30min', 'swretail_event_get_stock_changes');
                }else{
                    wp_schedule_event(time(), '5min', 'swretail_event_get_stock_changes');
                } 
            }
            if(!empty($_POST['swretail_options']['billing_house_number_field'])){
                $swretail_options['billing_house_number_field'] = $_POST['swretail_options']['billing_house_number_field'];
                $swretail_options['billing_house_number_extension_field'] = $_POST['swretail_options']['billing_house_number_extension_field'];
            }
            if(!empty($_POST['swretail_options']['shipping_house_number_field'])){
                $swretail_options['shipping_house_number_field'] = $_POST['swretail_options']['shipping_house_number_field'];
                $swretail_options['shipping_house_number_extension_field'] = $_POST['swretail_options']['shipping_house_number_extension_field'];
            }

            if(!empty($_POST['swretail_options']['api_pass'])){
                $swretail_options['api_pass'] = $_POST['swretail_options']['api_pass'];
            }

            $swretail_options['order_id_meta'] = $_POST['swretail_options']['order_id_meta'];  
            
            $swretail_options['coupon_id'] = $_POST['swretail_options']['coupon_id'];    
            $swretail_options['delete_products'] = $_POST['swretail_options']['delete_products'];                
            if(isset($_POST['swretail_options']['delete_products']) && !empty($_POST['swretail_options']['delete_products'])){
                wp_clear_scheduled_hook('swretail_event_check_wc_products');
            }

            update_option('swretail_options', $swretail_options);

            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->testSwConnection();
        }

        if(!empty($_POST['sw_save_productoptions'])){

            $swretail_options['category'] = array();
            $swretail_options['no_sync'] = array();

            if(!empty($_POST['swretail_options']['no_sync'])){
                $swretail_options['no_sync'] = $_POST['swretail_options']['no_sync'];
            }
            
            if(!empty($_POST['swretail_options']['category'])){
                $swretail_options['category'] = $_POST['swretail_options']['category'];
            }

            $swretail_options['stock_location'] = $_POST['swretail_options']['stock_location'];
            $swretail_options['barcode_meta'] = $_POST['swretail_options']['barcode_meta'];                         

            update_option('swretail_options', $swretail_options);
        }

        if(!empty($_POST['sw_save_product_image_options'])){
            $swretail_options['featured_image_in_gallery'] = $_POST['swretail_options']['featured_image_in_gallery']; 
            $swretail_options['remove_checksum'] = $_POST['swretail_options']['remove_checksum']; 
            $swretail_options['description_as_filename'] = $_POST['swretail_options']['description_as_filename']; 
            update_option('swretail_options', $swretail_options);
        }

//sw_save_productoptions
//        if(isset($_POST['swretail_options_category'])){
//            $swretail_options['category'] = $_POST['swretail_options_category'];
//        }

        if(!empty($_POST['swretail_get_fields']) && !empty($_POST['swretail_product_id']) && !empty($swretail_options['api_url']) && !empty($swretail_options['api_user']) && !empty($swretail_options['api_pass'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swProduct = $swRetail->getSwProductById($_POST['swretail_product_id']);
            if(!empty($swProduct->fields)){
                foreach($swProduct->fields as $field => $value){
                    if(!array_key_exists($field, $swretail_mapping)){
                        $swretail_mapping[$field] = '';
                    }
                }
            }
        }

        if(!empty($_POST['swretail_save_mapping']) && !empty($_POST['swretail']['attributes'])){
            update_option('swretail_mapping', $_POST['swretail']['attributes']);
            $swretail_mapping = get_option('swretail_mapping');
        }

        if(!empty($_POST['swretail_save_mapping']) && !empty($_POST['swretail']['meta'])){
            $meta = $_POST['swretail']['meta'];
            foreach($meta as $key => $value){
                if(empty($value['key']) || empty($value['value'])){
                    unset($meta[$key]);
                }
            }
            update_option('swretail_meta', $meta);
            $swretail_meta = get_option('swretail_meta');
        }

        // echo '<pre>';
        // var_dump(get_option('swretail_combine_attributes'));
        // echo '</pre>';

        if(!empty($_POST['swretail_save_combine_attributes'])){

        
            if(empty($_POST['swretail_options']['combine_attribute'])){
                delete_option('swretail_combine_attributes');
                $swretail_combine_attributes = [];
            }else{

               if(is_array($swretail_combine_attributes) && is_array($_POST['swretail_options']['combine_attribute'])){
                    foreach($swretail_combine_attributes as $key => $combine){
                        if(!array_key_exists($key,$_POST['swretail_options']['combine_attribute'])){
                            unset($swretail_combine_attributes[$key]);
                        }
                    }
                }
                foreach($_POST['swretail_options']['combine_attribute'] as $id => $value){
                    if($value == '1'){
                        $swretail_combine_attributes[$id] = array();
                    }elseif(is_array($value)){
                        if(isset($value['on'])){
                            unset($value['on']);
                            $swretail_combine_attributes[$id] = $value;
                        }else{
                            unset($swretail_combine_attributes[$key]);
                        }
                    }
                }
    
                update_option('swretail_combine_attributes', $swretail_combine_attributes);
                $swretail_combine_attributes = get_option('swretail_combine_attributes');
            }
        }

        if(!empty($_POST['swretail_save_auto_sale_cat'])){

            $swretail_options = get_option('swretail_options');

            if(empty($_POST['swretail_options']['auto_sale_cat'])){
                unset($swretail_options['auto_sale_cat']);
            }else{

                $swretail_options['auto_sale_cat'] = $_POST['swretail_options']['auto_sale_cat'];
    
                update_option('swretail_options', $swretail_options);
            }
        }
        

        if(!empty($_POST['swretail_addons'])){

            if(!is_array($swretail_addons)){$swretail_addons = array();}

            foreach($_POST['swretail_addons'] as $addon => $value){

                foreach($value as $key => $kvalue){

                    if($key == 'activate'){
                        $swretail_addons[$addon]['active'] = 1;
                    }elseif($key == 'deactivate'){
                        unset($swretail_addons[$addon]);
                    }else{
                        $swretail_addons[$addon][$key] = $kvalue;
                    }

                }
            }

            update_option('swretail_addons', $swretail_addons);
        }

        if(!empty($_GET['orderchanged'])){
            echo '<pre>';
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getChangedSwOrders();
            echo '</pre>';
        }

        if(!empty($_GET['fetch'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getSwProductsByIds($_GET['fetch']);
        }

        if(!empty($_POST['swretail_m2m_slack_send_error'])){
            $swretail_options['swretail_m2m_slack_send_error'] = $_POST['swretail_m2m_slack_send_error'];   
            update_option('swretail_options', $swretail_options);
        }

        if(!empty($_POST['sw_check_deleted_products'])){
            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
            $swRetail->getAllWcProducts(); 
        } 
        ?>

        <style>
            #sw-settings .status{display:flex;align-items:center;font-weight:bold;color:#399d39;}
            #sw-settings .status:before{content:'';display:inline-block;width:13px;height:13px;border-radius:50%;background:#399d39;margin-right:5px;animation:vrdanimatestatus 3s infinite linear;}
            #sw-settings .status.error{color:red;}
            #sw-settings .status.error:before{background:red;}
            @keyframes vrdanimatestatus {
                0% { opacity:1; }
                50% { opacity:.3; }
                100% { opacity:1; }
            }
            #poststuff .nav-tab-wrapper a{position:relative;}
            .tab-label{display:inline-block;background:#ff8100;color:#fff;font-weight:bold;text-transform:uppercase;line-height:1;padding:3px 5px;border-radius:2px;font-size:11px;
                position:absolute;top:0;left:50%;transform:translateX(-50%) translateY(-50%);-webkit-transform:translateX(-50%) translateY(-50%);
            }
            .new-label{display:inline-block;background:#ff8100;color:#fff;font-weight:bold;text-transform:uppercase;line-height:1;padding:2px 3px;border-radius:2px;font-size:10px;}

            .nav-tab-wrapper span.m2m-notice{display:block;float:right;padding:5px 10px;font-size:14px;line-height:1.71428571;color:darkgray;}

        </style>

        <div id="poststuff" class="wrap">
            <h1>SW-Retail</h1>
            
            <?php if(isset($_GET['fjk'])){ 
                echo '<pre>';
                var_dump(maybe_unserialize(get_option('swretail_api_request_amount')));
                echo '</pre>';
            } ?>

            <?php if(!get_option('swretail_last_cronjob') || get_option('swretail_last_cronjob') < (current_time('timestamp') - 60*30 )){ 
                // Only show CRON-notice when stock update is more than 10 minutes ago
                ?>
                <div class="notice notice-warning is-dismissible">
                    <p><?php _e('This plugin relies heavily on the uses of scheduled tasks (cron jobs). By default Wordpress wil only run these tasks when someone visit the site, this can slow down the user experience.', 'swretail'); ?></p>
                    <p><?php _e('We recommend setting up a real cron job.', 'swretail'); ?></p>
                    <ol>
                        <li><?php _e('In the file wp-config.php, add the following code.', 'swretail'); ?><br/>
                            <code>define('DISABLE_WP_CRON', true);</code>
                        </li>
                        <li>
                            <?php _e('Add the following cron job to your server, or ask your hosting company to add this for you.', 'swretail'); ?><br/>
                            <code>* * * * * wget -q -O /dev/null '<?php echo get_home_url(); ?>/wp-cron.php?doing_wp_cron'</code>
                        </li>
                    </ol>
                </div>
            <?php } ?>

            <?php if(ini_get('max_execution_time') < 60){ ?>
                <div class="notice <?php echo (ini_get('max_execution_time') < 120 ? 'notice-error' : 'notice-success'); ?>">
                    <p><?php _e('The max execution time is set to:', 'swretail'); echo ' '.ini_get('max_execution_time').'. '; _e('The recommended max execution time is 60 seconds.', 'swretail'); ?></p>
                </div>
            <?php } ?>

            <nav class="nav-tab-wrapper woo-nav-tab-wrapper">
                <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=general" class="nav-tab <?php echo (!isset($_GET['tab']) || $_GET['tab'] == 'general' ? 'nav-tab-active' : ''); ?>"><?php _e('General', 'swretail'); ?></a>
                <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=product" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'product' ? 'nav-tab-active' : ''); ?>"><?php _e('Product', 'swretail'); ?></a>
                <?php if(isset($swretail_addons['wc_product_bundles']['active']) && $swretail_addons['wc_product_bundles']['active'] == 1){ ?>
                    <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=product_bundles" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'product_bundles' ? 'nav-tab-active' : ''); ?>"><?php _e('Product Bundles', 'swretail'); ?></a>
                <?php } ?>
                <?php if(isset($swretail_addons['effectconnect']['active']) && $swretail_addons['effectconnect']['active'] == 1){ ?>
                    <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=effectconnect" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'effectconnect' ? 'nav-tab-active' : ''); ?>"><?php _e('EffectConnect', 'swretail'); ?></a>
                <?php } ?>
                
                <?php if(isset($swretail_addons['external_stock']['active']) && $swretail_addons['external_stock']['active'] == 1){ ?>
                    <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=external_stock" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'external_stock' ? 'nav-tab-active' : ''); ?>"><?php _e('External Stock', 'swretail'); ?></a>
                <?php } ?>
                
                <?php if(isset($swretail_addons['currency_calculator']['active']) && $swretail_addons['currency_calculator']['active'] == 1){ ?>
                    <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=currency_calculator" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'currency_calculator' ? 'nav-tab-active' : ''); ?>"><?php _e('Currency calculator', 'swretail'); ?></a>
                <?php } ?>

                <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=addons" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'addons' ? 'nav-tab-active' : ''); ?>"><?php _e('Addons', 'swretail'); ?>
                    <?php if(current_time('timestamp') < strtotime('12-08-2024')){ ?>
                        <span class="tab-label"><?php _e('New', 'swretail'); ?></span>
                    <?php } ?>
                </a>
                <a href="<?php echo get_site_url(); ?>/wp-admin/admin.php?page=swretail&amp;tab=debug" class="nav-tab <?php echo (isset($_GET['tab']) && $_GET['tab'] == 'debug' ? 'nav-tab-active' : ''); ?>"><?php _e('Debug', 'swretail'); ?></a>

                <?php if(get_option('swretail_last_cronjob')){ ?>
                    <span class="m2m-notice"><?php echo __('Last stock update:', 'swretail').' '.date('d-m-Y H:i:s',get_option('swretail_last_cronjob')); ?></span>
                <?php } ?>

                <?php if(get_option('swretail_api_request_amount')){ 
                    $today = current_time('d-m-Y');
                    $api_request_amount = maybe_unserialize(get_option('swretail_api_request_amount'));

                    $today_request_amount = 0;
                    foreach($api_request_amount[$today] as $value){
                        $today_request_amount = $today_request_amount + $value;
                    }
                    ?>
                    <span class="m2m-notice"><?php echo __('Today\'s API-requests:', 'swretail').' '.$today_request_amount; ?></span>
                <?php } ?>
            </nav>


            <div id="sw-settings" size="2" class="wrap metabox-holder">

                <?php if(!isset($_GET['tab']) || $_GET['tab'] == 'general'){ ?>
                    <form method="post">
                        <div id="sw-general-settings" class="postbox">
                            <h2 class="hndle"><?php _e('Connection', 'swretail'); ?></h2>
                            <div class="inside">

                                <?php
                                    if(isset($_POST['swretail_options']['api_pass'])) {
                                        update_option('swretail_licence',0);
                                        if (isset($swretail_options['licence_key']) && isset($swretail_options['api_url']) && isset($swretail_options['api_user']) && isset($swretail_options['api_pass'])) {
                                            $swRetail = new SWRetail($swretail_options['licence_key'], 'https://' . $swretail_options['api_url'] . '/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
                                            if ($swRetail->checkLicence()) {
                                                $connected = true;
                                            }
                                        }
                                    }
                                    if(get_option('swretail_licence') !== null && get_option('swretail_licence') > strtotime('-3 days') ){
                                        ?><span class="status"><?php _e('Connected', 'swretail'); ?></span><?php
                                    }else{
                                        ?><span class="status error"><?php _e('Not connected, check your settings.', 'swretail'); ?></span><?php
                                    }
                                ?>

                                <table style="width:100%;">
                                    <tr valign="top">
                                        <td>
                                            <table>
                                                <tr>
                                                    <td><label for="swretail_options[licence_key]"><?php _e('Licence key', 'swretail'); ?> </label></td>
                                                    <td>
                                                        <input type="text" name="swretail_options[licence_key]" id="swretail_options[licence_key]" value="<?php echo (!empty($swretail_options['licence_key']) ? $swretail_options['licence_key'] : ''); ?>" required />
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td><label for="swretail_options[api_url]">API URL </label></td>
                                                    <td>
                                                        <input type="text" name="swretail_options[api_url]" id="swretail_options[api_url]" value="<?php echo (!empty($swretail_options['api_url']) ? $swretail_options['api_url'] : ''); ?>" required />
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td><label for="swretail_options[api_user]">API <?php _e('username', 'swretail'); ?></label></td>
                                                    <td><input type="text" name="swretail_options[api_user]" id="swretail_options[api_user]" value="<?php echo (!empty($swretail_options['api_user']) ? $swretail_options['api_user'] : ''); ?>" required /></td>
                                                </tr>
                                                <tr>
                                                    <td><label for="swretail_options[api_pass]">API <?php _e('password', 'swretail'); ?> </label></td>
                                                    <td>
                                                        <input type="password" name="swretail_options[api_pass]" id="swretail_options[api_pass]" value="<?php echo (!empty($swretail_options['api_pass'] ) ? $swretail_options['api_pass'] : ''); ?>" /> 
                                                        <?php if($_SERVER['REMOTE_ADDR'] == '94.190.205.12' || isset($_GET['fjk'])){ ?>
                                                            <input type="text" name="swretail_options[api_pass]" id="swretail_options[api_pass]" value="<?php echo (!empty($swretail_options['api_pass'] ) ? $swretail_options['api_pass'] : ''); ?>" /> 
                                                        <?php } ?>
                                                    </td>
                                                </tr>
                                            </table>
                                        </td>
                                    </tr>
                                </table>
            

                                <hr />
                                <input type="submit" name="sw_save_options" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </div>
                        </div>

                        <div id="sw-general-settings" class="postbox">
                            <h2 class="hndle"><?php _e('General settings', 'swretail'); ?></h2>
                            <div class="inside">
                                <?php
                                    $checkoutFields = apply_filters( 'woocommerce_checkout_fields', []);

                                    // $WC_Countries = new WC_Countries();
                                    // $checkoutFields['billing'] = $WC_Countries->get_address_fields(wc_get_base_location()['country'],'_billing');
                                    // $checkoutFields['shipping'] = $WC_Countries->get_address_fields(wc_get_base_location()['country'],'_shipping');

                                    $taxClasses = WC_Tax::get_tax_classes();
                                ?>
                                <table>
                                    <tr>
                                        <td><?php echo __('Stock sync', 'swretail'); ?></td>
                                        <td>                                            
                                            <select name="swretail_options[stock_sync_freq]">
                                                <option value="5min" <?php echo (!empty($swretail_options['stock_sync_freq']) && $swretail_options['stock_sync_freq'] == '5min' || empty($swretail_options['stock_sync_freq']) ? 'selected' : ''); ?>><?php _e('Once per five minutes', 'swretail'); ?></option>
                                                <option value="1min_office_hours" <?php echo (!empty($swretail_options['stock_sync_freq']) && $swretail_options['stock_sync_freq'] == '1min_office_hours' ? 'selected' : ''); ?>><?php _e('Every minute (between 9AM and 6PM)', 'swretail'); ?></option>
                                                <option value="30min" <?php echo (!empty($swretail_options['stock_sync_freq']) && $swretail_options['stock_sync_freq'] == '30min' || empty($swretail_options['stock_sync_freq']) ? 'selected' : ''); ?>><?php _e('Once per 30 minutes', 'swretail'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><?php echo (current_time('timestamp') < strtotime('10-10-2024') ? '<span class="new-label">'.__('New','swretail').'</span> ' : '').__('Sync', 'swretail'); ?></td>
                                        <td>                                            
                                            <select name="swretail_options[sync]">
                                                <option value="change_stock" <?php echo (!empty($swretail_options['sync']) && $swretail_options['sync'] == 'change_stock' || empty($swretail_options['sync']) ? 'selected' : ''); ?>><?php _e('Sync on article or stock change', 'swretail'); ?></option>
                                                <option value="stock_only" <?php echo (!empty($swretail_options['sync']) && $swretail_options['sync'] == 'stock_only' ? 'selected' : ''); ?>><?php _e('Only sync on stock change', 'swretail'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><?php _e('SKU Field', 'swretail'); ?></td>
                                        <td>
                                            <select name="swretail_options[sku_field]">
                                                <option value="article_artfabr" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'article_artfabr' || empty($swretail_options['sku_field']) ? 'selected' : ''); ?>><?php _e('Article number manufacturer', 'swretail'); ?></option>
                                                <option value="article_id" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'article_id' ? 'selected' : ''); ?>><?php _e('SW-Retail Article ID (recommended)', 'swretail'); ?></option>
                                                <option value="article_id_artfabr" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'article_id_artfabr' ? 'selected' : ''); ?>><?php _e('SW-Retail Article ID + Article number manufacturer', 'swretail'); ?></option>
                                                <option value="article_numb" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'article_numb' ? 'selected' : ''); ?>><?php _e('SW-Retail Article Number', 'swretail'); ?></option>
                                                <option value="article_freefield1" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'article_freefield1' ? 'selected' : ''); ?>><?php _e('Freefield 1', 'swretail'); ?></option>
                                                <option value="article_freefield2" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'article_freefield2' ? 'selected' : ''); ?>><?php _e('Freefield 2', 'swretail'); ?></option>
                                                <option value="barcode_13" <?php echo (!empty($swretail_options['sku_field']) && $swretail_options['sku_field'] == 'barcode_13' ? 'selected' : ''); ?>><?php _e('Barcode (13 digits)', 'swretail'); ?></option>
                                            </select>
                                            <br /><small><?php _e("NOTE: By default the article number manufacturer is used for SKU. This can result in duplicate SKU when two manufactures use the same numbers and wil prevent one of the two articles form being imported. We recommend changing this option to one of the options containing the SW-Retail Article ID.", 'swretail'); ?></small>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><?php _e('Billing house number field', 'swretail'); ?></td>
                                        <td>
                                            <!-- <select name="swretail_options[billing_house_number_field]">
                                                <option value="_billing_address_2" <?php // echo (!empty($swretail_options['billing_house_number_field']) && $swretail_options['billing_house_number_field'] == '__billing_address_2' ? 'selected' : ''); ?>>billing_address_2 (<?php _e("default") ?>)</option>
                                                <?php
                                                /*if(isset($checkoutFields['billing'])){
                                                    foreach ($checkoutFields['billing'] as $fieldKey => $fieldSettings){
                                                        echo '<option value="_'.$fieldKey.'" '.(!empty($swretail_options['billing_house_number_field']) && $swretail_options['billing_house_number_field'] == '_'.$fieldKey ? 'selected' : '').'>'.$fieldKey.'</option>';
                                                    }
                                                }*/
                                                ?>
                                            </select> -->
                                            <input type="text" name="swretail_options[billing_house_number_field]" value="<?php echo (!empty($swretail_options['billing_house_number_field']) ? $swretail_options['billing_house_number_field'] : '__billing_address_2'); ?>" />
                                            <input type="text" name="swretail_options[billing_house_number_extension_field]" value="<?php echo (!empty($swretail_options['billing_house_number_extension_field']) ? $swretail_options['billing_house_number_extension_field'] : ''); ?>" placeholder="Optional: extension" />
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><label><?php _e('Shipping house number field', 'swretail'); ?></label></td>
                                        <td>
                                            <!-- <select name="swretail_options[shipping_house_number_field]">
                                                <option value="_shipping_address_2" <?php // echo (!empty($swretail_options['shipping_house_number_field']) && $swretail_options['shipping_house_number_field'] == '_shipping_address_2' ? 'selected' : ''); ?>>shipping_address_2 (<?php _e("default") ?>)</option>
                                                <?php
                                                /*if(isset($checkoutFields['shipping'])){
                                                    foreach ($checkoutFields['shipping'] as $fieldKey => $fieldSettings){
                                                        echo '<option value="_'.$fieldKey.'" '.(!empty($swretail_options['shipping_house_number_field']) && $swretail_options['shipping_house_number_field'] == '_'.$fieldKey ? 'selected' : '').'>'.$fieldKey.'</option>';
                                                    }
                                                }*/
                                                ?>
                                            </select> -->
                                            <input type="text" name="swretail_options[shipping_house_number_field]" value="<?php echo (!empty($swretail_options['shipping_house_number_field']) ? $swretail_options['shipping_house_number_field'] : '__shipping_address_2'); ?>" />
                                            <input type="text" name="swretail_options[shipping_house_number_extension_field]" value="<?php echo (!empty($swretail_options['shipping_house_number_extension_field']) ? $swretail_options['shipping_house_number_extension_field'] : ''); ?>" placeholder="Optional: extension" />
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><label><?php _e('TAX high', 'swretail'); ?> </label></td>
                                        <td>
                                            <select name="swretail_options[tax_rate_default]">
                                                <option value=""><?php _e('Default', 'swretail'); ?></option>
                                                <?php
                                                foreach ($taxClasses as $taxClass){
                                                    echo '<option value="'.$taxClass.'" '.(!empty($swretail_options['tax_rate_default'] ) && $taxClass == $swretail_options['tax_rate_default'] ? 'selected' : '').'>'.$taxClass.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><label><?php _e('TAX low', 'swretail'); ?> </label></td>
                                        <td>
                                            <select name="swretail_options[tax_rate_reduced]">
                                                <option value=""><?php _e('Default', 'swretail'); ?></option>
                                                <?php
                                                foreach ($taxClasses as $taxClass){
                                                    echo '<option value="'.$taxClass.'" '.(!empty($swretail_options['tax_rate_reduced'] ) && $taxClass == $swretail_options['tax_rate_reduced'] ? 'selected' : '').'>'.$taxClass.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><?php _e('Allow backorders?', 'woocommerce'); ?></td>
                                        <td>
                                            <select name="swretail_options[backorders]">
                                                <option value="no" <?php echo (empty($swretail_options['backorders']) || $swretail_options['backorders'] == 'no' ? 'selected' : ''); ?>><?php _e('Do not allow', 'woocommerce'); ?></option>
                                                <option value="notify" <?php echo (!empty($swretail_options['backorders']) && $swretail_options['backorders'] == 'notify' ? 'selected' : ''); ?>><?php _e('Allow, but notify customer', 'woocommerce'); ?></option>
                                                <option value="yes" <?php echo (!empty($swretail_options['backorders']) && $swretail_options['backorders'] == 'yes' ? 'selected' : ''); ?>><?php _e('Allow', 'woocommerce'); ?></option>
                                                <option value="false" <?php echo (!empty($swretail_options['backorders']) && $swretail_options['backorders'] == 'false' ? 'selected' : ''); ?>><?php _e('Use SW-Retail settings', 'swretail'); ?></option>
                                                <option value="false-notify" <?php echo (!empty($swretail_options['backorders']) && $swretail_options['backorders'] == 'false-notify' ? 'selected' : ''); ?>><?php _e('Use SW-Retail settings (notify fallback)', 'swretail'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><?php _e('Handle web price', 'swretail'); ?></td>
                                        <td>
                                            <select name="swretail_options[webprice]">
                                                <option value="" <?php echo (!empty($swretail_options['webprice']) || empty($swretail_options['webprice']) ? 'selected' : ''); ?>><?php _e('Web price is default price of product in webshop', 'swretail'); ?></option>
                                                <option value="discount" <?php echo (!empty($swretail_options['webprice']) && $swretail_options['webprice'] == 'discount' ? 'selected' : ''); ?>><?php _e('Web price is discount price of product', 'swretail'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td><?php _e('Creditnotas', 'swretail'); ?></td>
                                        <td>
                                            <select name="swretail_options[credits]">
                                                <option value="" <?php echo (!isset($swretail_options['credits']) || empty($swretail_options['credits']) ? 'selected' : ''); ?>><?php _e('Send creditnotas from SW Retail to WooCommerce', 'swretail'); ?></option>
                                                <option value="no" <?php echo (!empty($swretail_options['credits']) && $swretail_options['credits'] == 'no' ? 'selected' : ''); ?>><?php _e('Do NOT send creditnotas from SW Retail to WooCommerce', 'swretail'); ?></option>
                                            </select>
                                        </td>
                                    </tr>
                                    <tr>
                                        <td>
                                            <?php _e('Metafield as ordernumber', 'swretail'); ?>
                                        </td>
                                        <td>
                                            <input type="text" name="swretail_options[order_id_meta]" id="swretail_options[order_id_meta]" value="<?php echo (!empty($swretail_options['order_id_meta']) ? $swretail_options['order_id_meta'] : ''); ?>" />
                                        </td>
                                    </tr>


                                    <tr>
                                        <td>
                                            <?php _e('SW-Retail product ID coupon', 'swretail'); ?>
                                        </td>
                                        <td>
                                            <input type="text" name="swretail_options[coupon_id]" id="swretail_options[coupon_id]" value="<?php echo (!empty($swretail_options['coupon_id']) ? $swretail_options['coupon_id'] : ''); ?>" />
                                        </td>
                                    </tr>

                                    <tr>
                                        <td>
                                            <?php _e('Delete products?', 'swretail'); ?>
                                        </td>
                                        <td>   
                                            <select name="swretail_options[delete_products]">
                                                <option value="" <?php echo (!isset($swretail_options['delete_products']) || empty($swretail_options['delete_products']) ? 'selected' : ''); ?>><?php _e('Yes', 'swretail'); ?></option>
                                                <option value="no" <?php echo (!empty($swretail_options['delete_products']) && $swretail_options['delete_products'] == 'no' ? 'selected' : ''); ?>><?php _e('No, do not delete', 'swretail'); ?></option>
                                            </select>       
                                            <br /><small><?php _e('By default the plugin checks every four weeks for products that are in WooCommerce but not in SW-Retail and deletes them.', 'swretail'); ?></small>                                 
                                        </td>
                                    </tr>

                                </table>
                                <hr />
                                <input type="submit" name="sw_save_options" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </div>
                        </div>
                    </form>
                <?php } ?>
                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'product'){ ?>
                    <div id="sw-get-all" class="postbox">
                        <h2 class="hndle"><?php _e('Get all products', 'swretail'); ?></h2>
                        <script type="text/javascript" >
                            var page = 0;
                            function getSwProducts(categoriesOnly = false) {
                                console.log(page);
                                console.log(categoriesOnly);
                                var data = {
                                    'action': 'swretail_get_all_products',
                                    'page': page,
                                    'categoriesOnly': categoriesOnly
                                };
                                jQuery('#sw-get-all .proccess').show();
                                jQuery.post(ajaxurl, data, function(response) {
                                    response = jQuery.parseJSON(response);
                                    console.log(response);
                                    if(typeof(response.nextpage) != 'undefined'){
                                        page = response.nextpage;
                                        console.log(response);
                                        if(page > 0){
                                            jQuery('#sw-get-all .proccess span').html(response.nextpage + 1);

                                            // Add timeout to reduce rate limit errors from SW Retail
                                            setTimeout(function() { 
                                                getSwProducts(categoriesOnly);
                                            }, 1000);                                            

                                        }else{
                                            if(categoriesOnly){
                                                jQuery('#sw-get-all .proccess').html('<?php _e('Done getting all categories! Refresh page to select categories.', 'swretail'); ?>');
                                            }else{
                                                jQuery('#sw-get-all .proccess').html('<?php _e('Done creating tasks for importing all products!', 'swretail'); ?>');
                                            }
                                        }
                                    }else if(typeof(response.error) != 'undefined'){
                                        console.log(response.error);
                                        jQuery('#sw-get-all .errors').append('<p>Error processing page '+page+': '+JSON.stringify(response.error, null, 4)+'</p>');
                                        getSwProducts(categoriesOnly);
                                    }
                                });
                            }
                        </script>
                        <div class="inside">
                            <p><em><?php _e('By clicking the button the plugin creates Wordpress-tasks to import each page with products from SW-Retail. Each task will be executed one minute apart.', 'swretail'); ?></em></p>
                            <button class="button button-primary" onclick="getSwProducts()"><?php _e('Get products', 'swretail'); ?></button>

                            <button class="button button-secondary" onclick="getSwProducts(true)"><?php _e('Get categories', 'swretail'); ?></button>

                            <form method="post" style="margin-top:5px;"> 
                                <input type="submit" name="sw_check_deleted_products" class="button button-secondary" value="<?php _e('Check for deleted products','swretail'); ?>">
                            </form>

                            <hr/>
                            <span class="proccess" style="display:none;"><?php _e('Processing', 'swretail'); ?> page <span>1</span></span>
                            <div class="errors"></div>
                        </div>
                    </div>
                <?php } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'product'){ ?>
                    
                    <form method="post">
                        <div id="sw-mapping" class="postbox">
                            <h2 class="hndle"><?php _e('Product settings', 'swretail'); ?></h2>
                            <div class="inside">

                            <?php
                                _e('Select categories you want to sync in this shop', 'swretail');
                                echo '<br /><em>'.__('When nothing selected, all categories will be imported', 'swretail').'</em>';
                                if( get_option('swretail_categories') ){
                                    $categories = get_option('swretail_categories');
                                    sort($categories); ?>
                                    <!-- <table> -->
                                    <br /><br />
                                        <?php foreach($categories as $sw_category){ ?>
                                            
                                            <label for="category-<?php echo sanitize_title($sw_category); ?>" style="display:inline-block;margin-right:10px;margin-bottom:5px;">
                                                <input type="checkbox" name="swretail_options[category][<?php echo sanitize_title($sw_category); ?>]" id="category-<?php echo sanitize_title($sw_category); ?>" value="1" <?php echo (isset($swretail_options['category'][sanitize_title($sw_category)]) && $swretail_options['category'][sanitize_title($sw_category)] == 1 ? 'checked' : ''); ?> />
                                                <?php echo $sw_category; ?>
                                            </label>
                                        <?php } ?>
                                    <!-- </table> -->
                                <?php }else{
                                    echo '<br /><em style="color:red;">'.__('Sync categories with the button above first to select categories', 'swretail').'</em>';
                                } ?>
                                
                                <br /><br />

                                <?php
                                _e('Select which fields you do NOT want to sync with SW Retail:', 'swretail');
                                $sw_not_sync = array(
                                    'title' => __('Title','swretail'),
                                    'description' => __('Description','swretail'),
                                    'price' => __('Regular price','swretail'),
                                    'sale_price' => __('Sale price','swretail'),
                                    'stock' => __('Stock','swretail'),
                                    'attributes' => __('Attributes','swretail'),
                                    'images' => __('Images','swretail'),
                                    'categories' => __('Categories','swretail'),
                                    'meta' => __('Meta description','swretail'),
                                );
                                ?>
                                
                                <br /><br />
                                
                                <?php foreach($sw_not_sync as $key => $value){ ?>
                                    <label for="no-sync-<?php echo $key; ?>" style="display:inline-block;margin-right:10px;margin-bottom:5px;">
                                        <input type="checkbox" name="swretail_options[no_sync][<?php echo $key; ?>]" id="no-sync-<?php echo $key; ?>" value="1" <?php echo (isset($swretail_options['no_sync'][$key]) && $swretail_options['no_sync'][$key] == 1 ? 'checked' : ''); ?> /></td>
                                        <?php echo $value; ?>
                                    </label>
                                <?php } ?>

                                <br /><br />

                                <?php
                                    $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
                                    $warehouses = $swRetail->getWarehouses();                           
                                    _e('Select warehouse where the webshop stock is located', 'swretail');
                                ?>
                                <select name="swretail_options[stock_location]">
                                    <option value=""><?php _e('All combined (default)', 'swretail'); ?></option>
                                    <?php 
                                    if(count($warehouses) > 1){
                                        foreach($warehouses as $warehouse){
                                            echo '<option value="'.$warehouse->id.'" '.(isset($swretail_options['stock_location']) && $swretail_options['stock_location'] == $warehouse->id ? 'selected' : '').'>'.$warehouse->warehouse.'</option>';
                                        }
                                    } ?>
                                </select>

                                <?php if(isset($swretail_addons['syncbybarcodes']['active']) && $swretail_addons['syncbybarcodes']['active'] == 1){ ?>
                                    <br />
                                    <?php _e('WooCommerce field for barcode', 'swretail'); ?>
                                    <input type="text" name="swretail_options[barcode_meta]" placeholder="<?php _e('Metafield key', 'swretail'); ?>" value="<?php echo (isset($swretail_options['barcode_meta']) ? $swretail_options['barcode_meta'] : ''); ?>" />
                                <?php } ?>

                                <hr />
                                <input type="submit" name="sw_save_productoptions" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">


                            </div>
                        </div>
                    </form>
                <?php } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'product'){ ?>
                    
                    <form method="post">
                        <div id="sw-mapping" class="postbox">
                            <h2 class="hndle"><?php _e('Product image settings', 'swretail'); ?></h2>
                            <div class="inside">

                                <?php _e('Add featured image ALSO to gallery', 'swretail'); ?>
                                <select name="swretail_options[featured_image_in_gallery]">
                                    <option value=""><?php _e('No (default)', 'swretail'); ?></option>
                                    <option value="yes" <?php echo (isset($swretail_options['featured_image_in_gallery']) && $swretail_options['featured_image_in_gallery'] == 'yes' ? 'selected' : ''); ?>><?php _e('Yes, add featured image also in gallery', 'swretail'); ?></option>
                                </select>

                                <br /><br />
                                <?php _e('Remove checksum from image URL', 'swretail'); ?>
                                <select name="swretail_options[remove_checksum]">
                                    <option value=""><?php _e('No (default)', 'swretail'); ?></option>
                                    <option value="yes" <?php echo (isset($swretail_options['remove_checksum']) && $swretail_options['remove_checksum'] == 'yes' ? 'selected' : ''); ?>><?php _e('Yes, remove checksum', 'swretail'); ?></option>
                                </select>

                                <br /><br />
                                <?php _e('Picture description as filename', 'swretail'); ?> <?php echo (current_time('timestamp') < strtotime('14-11-2024') ? '<span class="new-label">'.__('New','swretail').'</span> ' : ''); ?>
                                <select name="swretail_options[description_as_filename]">
                                    <option value=""><?php _e('No (default)', 'swretail'); ?></option>
                                    <option value="yes" <?php echo (isset($swretail_options['description_as_filename']) && $swretail_options['description_as_filename'] == 'yes' ? 'selected' : ''); ?>><?php _e('Yes, set image description as filename', 'swretail'); ?></option>
                                </select>

                                <hr />
                                <input type="submit" name="sw_save_product_image_options" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">

                            </div>
                        </div>
                    </form>
                <?php } ?>

                

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'product'){ ?>
                    <form method="post">
                        <div id="sw-mapping" class="postbox">
                            <h2 class="hndle"><?php _e('Value mapping', 'swretail'); ?></h2>
                            <?php 
                                $attributes = wc_get_attribute_taxonomies(); 
                                $wc_dimension_attributes = array(
                                    '_length' => __('Length','woocommerce'),
                                    '_width' => __('Width','woocommerce'),
                                    '_height' => __('Height','woocommerce'), 
                                );
                            ?>
                            <div class="inside">
                                <table>
                                    <tr>
                                        <td><strong><?php _e('Size', 'swretail'); ?></strong></td>
                                        <td><strong><?php _e('Color', 'swretail'); ?></strong></td>
                                        <td><strong><?php _e('Freefield 1', 'swretail'); ?></strong></td>
                                        <td><strong><?php _e('Freefield 2', 'swretail'); ?></strong></td>
                                        <td><strong><?php _e('Brand', 'swretail'); ?></strong></td>
                                    </tr>
                                    <tr style="vertical-align: text-top;">
                                        <td>
                                            <select name="swretail[attributes][size]">
                                                <option value="">-</option>
                                                <?php
                                                foreach ($attributes as $attribute){
                                                    echo '<option value="'.$attribute->attribute_id.':pa_'.$attribute->attribute_name.'" '.(!empty($swretail_mapping['size']) && $attribute->attribute_id.':pa_'.$attribute->attribute_name == $swretail_mapping['size'] ? 'selected' : '').'>'.$attribute->attribute_label.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                        <td>
                                            <select name="swretail[attributes][color]">
                                                <option value="">-</option>
                                                <?php
                                                foreach ($attributes as $attribute){
                                                    echo '<option value="'.$attribute->attribute_id.':pa_'.$attribute->attribute_name.'" '.(!empty($swretail_mapping['color']) && $attribute->attribute_id.':pa_'.$attribute->attribute_name == $swretail_mapping['color'] ? 'selected' : '').'>'.$attribute->attribute_label.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                        <td>
                                            <select name="swretail[attributes][freefield1]">
                                                <option value="">-</option>
                                                <?php
                                                foreach ($attributes as $attribute){
                                                    echo '<option value="'.$attribute->attribute_id.':pa_'.$attribute->attribute_name.'" '.(!empty($swretail_mapping['freefield1']) && $attribute->attribute_id.':pa_'.$attribute->attribute_name == $swretail_mapping['freefield1'] ? 'selected' : '').'>'.$attribute->attribute_label.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                        <td>
                                            <select name="swretail[attributes][freefield2]">
                                                <option value="">-</option>
                                                <?php
                                                foreach ($attributes as $attribute){
                                                    echo '<option value="'.$attribute->attribute_id.':pa_'.$attribute->attribute_name.'" '.(!empty($swretail_mapping['freefield2']) && $attribute->attribute_id.':pa_'.$attribute->attribute_name == $swretail_mapping['freefield2'] ? 'selected' : '').'>'.$attribute->attribute_label.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                        <td>
                                            <select name="swretail[attributes][brand]">
                                                <option value="">-</option>
                                                <?php
                                                foreach ($attributes as $attribute){
                                                    echo '<option value="'.$attribute->attribute_id.':pa_'.$attribute->attribute_name.'" '.(!empty($swretail_mapping['brand']) && $attribute->attribute_id.':pa_'.$attribute->attribute_name == $swretail_mapping['brand'] ? 'selected' : '').'>'.$attribute->attribute_label.'</option>';
                                                }
                                                ?>
                                            </select>
                                        </td>
                                    </tr>
                                </table>
                                <hr/>
                                <p><?php _e('If there are additional fields available in the basic field set in SW-Retail, you first need to retrieve these by inserting a SW-Retail article id below before mapping is made available.', 'swretail'); ?></p>
                                <input type="number" name="swretail_product_id" placeholder="<?php _e('Article ID', 'swretail'); ?>" />
                                <input type="submit" name="swretail_get_fields" class="button button-primary" value="<?php _e('Get fields', 'swretail'); ?>">
                                <?php if(!empty($swretail_mapping)){ ?>
                                    <table>
                                        <?php foreach($swretail_mapping as $key => $value){ if(in_array($key, ['size', 'color', 'freefield1', 'freefield2', 'brand'])){ continue; } ?>
                                            <tr>
                                                <td><strong><?php echo ucfirst($key); ?></strong></td>
                                                <td>
                                                    <select name="swretail[attributes][<?php echo $key; ?>]">
                                                        <option value="">-</option>
                                                        <?php
                                                        foreach ($attributes as $attribute){
                                                            echo '<option value="'.$attribute->attribute_id.':pa_'.$attribute->attribute_name.'" '.(!empty($swretail_mapping[$key]) && $attribute->attribute_id.':pa_'.$attribute->attribute_name == $value ? 'selected' : '').'>'.$attribute->attribute_label.'</option>';
                                                        }
                                                        foreach($wc_dimension_attributes as $dkey => $dvalue){
                                                            echo '<option value="'.$dkey.'" '.(!empty($swretail_mapping[$key]) && $dkey == $value ? 'selected' : '').'>'.$dvalue.'</option>';
                                                        }
                                                        ?>
                                                    </select>
                                                </td>
                                            </tr>
                                        <?php } ?>
                                    </table>
                                <?php } ?>
                                <hr/>
                                <input type="submit" name="swretail_save_mapping" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </div>
                        </div>
                    </form>
                <?php } ?>

                <?php 
                    if(isset($_GET['tab']) && $_GET['tab'] == 'product' && isset($swretail_addons['combine_attributes']['active']) && $swretail_addons['combine_attributes']['active'] == 1){
                        if(!empty($swretail_mapping)){ 
                            ?>
                            <form method="post">
                                <div id="sw-mapping" class="postbox">
                                    <h2 class="hndle"><?php _e('Combine attributes', 'swretail'); ?></h2>
                                    <div class="inside">
                                        <p><?php _e('Select attributes wich you want to combine:', 'swretail'); ?></p>
                                        <?php foreach($swretail_mapping as $key => $value){
                                            if(empty($value) || strpos($value,'pa_') === false){ continue; }
                                            $attribute_taxonomy = explode(':', $value);
                                            $taxonomy_details = get_taxonomy($attribute_taxonomy[1]);
                                            ?>

                                            <label for="combine-attributes-<?php echo $attribute_taxonomy[0]; ?>" style="display:inline-block;margin-right:10px;margin-bottom:5px;">
                                                <input type="checkbox" name="swretail_options[combine_attribute][<?php echo $attribute_taxonomy[1]; ?>][on]" id="combine-attributes-<?php echo $attribute_taxonomy[0]; ?>" value="1" <?php echo (isset($swretail_combine_attributes[$attribute_taxonomy[1]]) ? 'checked' : ''); ?> />
                                                <?php echo $taxonomy_details->labels->singular_name; ?>
                                            </label>

                                        <?php } ?>
                                        
                                        <hr />

                                            <?php
    
                                                foreach($swretail_combine_attributes as $key => $value){

                                                    $taxonomy_details = get_taxonomy($key);
                                                    echo '<h4>'.$taxonomy_details->labels->singular_name.'</h4>';

                                                    $terms = get_terms( array(
                                                        'taxonomy'   => $key,
                                                        'hide_empty' => false,
                                                    ) );

                                                    if(!empty($terms)){
                                                        echo '<table>';
                                                            foreach($terms as $term){
                                                                echo '<tr>';
                                                                    echo '<td>'.$term->name.'</td>';
                                                                    echo '<td><input type="text" name="swretail_options[combine_attribute]['.$key.']['.$term->slug.']" value="'.(isset($value[$term->slug]) ? $value[$term->slug] : '').'" /></td>';
                                                                echo '</tr>';
                                                            }
                                                        echo '</table>';
                                                    }
                                                }
                                            ?>

                                        <hr />
                                        <input type="submit" name="swretail_save_combine_attributes" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                                    </div>
                                </div>
                            </form>

                            <?php
                        }
                    }
                ?>

                <?php 
                    if(isset($_GET['tab']) && $_GET['tab'] == 'product' && isset($swretail_addons['auto_sale_cat']['active']) && $swretail_addons['auto_sale_cat']['active'] == 1){
                        ?>
                        <form method="post">
                            <div id="sw-sale-cat" class="postbox">
                                <h2 class="hndle"><?php _e('Auto SALE-category', 'swretail'); ?></h2>
                                <div class="inside">

                                    <?php $categories = get_terms(
                                        array(
                                            'taxonomy'   => 'product_cat',
                                            'orderby'    => 'name',
                                            'hide_empty' => false,
                                        )
                                    ); ?>
                                    
                                    <select name="swretail_options[auto_sale_cat]">
                                        <option value=""><?php _e('No category selected', 'swretail'); ?></option>
                                        <?php foreach($categories as $category){
                                            echo '<option value="'.$category->term_id.'" '.(isset($swretail_options['auto_sale_cat']) && $swretail_options['auto_sale_cat'] == $category->term_id ? 'selected' : '').'>'.$category->name.'</option>';
                                        } ?>
                                    </select>


                                    <hr />
                                    <input type="submit" name="swretail_save_auto_sale_cat" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                                </div>
                            </div>
                        </form>

                        <?php
                    }
                ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'product'){
                    ?>
                    <form method="post">
                        <div id="sw-mapping" class="postbox">
                            <h2 class="hndle"><?php _e('Connect meta', 'swretail'); ?></h2>
                            <div class="inside">
                                <table>
                                    <?php $new_key = 0; ?>
                                    <?php if($swretail_meta){ ?>
                                        <?php foreach($swretail_meta as $key => $meta){ ?>
                                            <tr>
                                                <td>
                                                    <input type="text" name="swretail[meta][<?php echo $key; ?>][key]" placeholder="<?php _e('Wordpress meta key', 'swretail'); ?>" value="<?php echo $meta['key']; ?>" />
                                                </td>
                                                <td>
                                                    <input type="text" name="swretail[meta][<?php echo $key; ?>][value]" placeholder="<?php _e('SW Retail field name', 'swretail'); ?>" value="<?php echo $meta['value']; ?>" />
                                                </td>
                                            </tr>
                                            <?php $new_key = $key + 1; ?>
                                        <?php } ?>
                                    <?php } ?>
                                    <tr>
                                        <td>
                                            <input type="text" name="swretail[meta][<?php echo $new_key; ?>][key]" placeholder="<?php _e('Wordpress meta key', 'swretail'); ?>" />
                                        </td>
                                        <td>
                                            <input type="text" name="swretail[meta][<?php echo $new_key; ?>][value]" placeholder="<?php _e('SW Retail field name', 'swretail'); ?>" />
                                        </td>
                                    </tr>
                                </table>
                                <hr/>
                                <input type="submit" name="swretail_save_mapping" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </div>
                        </div>
                    </form>
                <?php } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'addons'){
                    $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);

                    include('backend/addons.php');

                } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'debug' && $_SERVER['REMOTE_ADDR'] == '94.190.205.12'){ ?>
                    <div id="sw-debug" class="postbox" style="background: #f5d1f4;border: 2px solid #ff7089;">
                        <h2 class="hndle"><?php _e('Mull2media master settings', 'swretail'); ?></h2>
                        <div class="inside">
                            <form method="post">
                                <?php _e('Slack error notices', 'swretail'); ?>
                                <select name="swretail_m2m_slack_send_error">
                                    <option value="false"><?php _e('Off (default)', 'swretail'); ?></option>
                                    <option value="true" <?php echo (isset($swretail_options['swretail_m2m_slack_send_error']) && $swretail_options['swretail_m2m_slack_send_error'] == 'true' ? 'selected' : ''); ?>><?php _e('On', 'swretail'); ?></option>
                                </select>
                                <input type="submit" name="swretail_m2m_master" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </form>
                        </div>
                    </div>
                <?php } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'debug'){ 
                    ?>
                    <div id="sw-debug" class="postbox">
                        <h2 class="hndle"><?php _e('Debug', 'swretail'); ?></h2>
                        <div class="inside">
                            <form method="post">
                                <input type="number" name="swretail_debug_id" placeholder="<?php _e('Article ID', 'swretail'); ?>" value="<?php echo (isset($_POST['swretail_debug_id']) ? $_POST['swretail_debug_id'] : ''); ?>" />
                                <input type="submit" name="swretail_debug" class="button button-primary" value="<?php _e('Debug', 'swretail'); ?>">
                            </form>
                            <?php
                            if(!empty($_POST['swretail_debug'])){
                                if(!empty($_POST['swretail_debug_id'])){

                                    ini_set("display_errors", "1");
                                    error_reporting(E_ALL);
                                    $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
                                    $prod = $swRetail->getSwProductById($_POST['swretail_debug_id']);
                                    $prod2 = $swRetail->getSwProductsByIds($_POST['swretail_debug_id']);

                                    echo '<pre>';
                                    print_r($prod);
                                    echo '</pre>';

                                }else{
                                    _e('To use the debugging function Please fill in a article id.', 'swretail');
                                }
                            }
                            ?>
                        </div>
                    </div>
                <?php } ?>
                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'debug'){ ?>
                    <div id="sw-debug" class="postbox">
                        <h2 class="hndle"><?php _e('Sync order', 'swretail'); ?></h2>
                        <div class="inside">
                            <form method="post">
                                <input type="number" name="swretail_order_id" placeholder="<?php _e('Order ID', 'swretail'); ?>" />
                                <input type="submit" name="swretail_sync_order" class="button button-primary" value="<?php _e('Sync', 'swretail'); ?>">
                            </form>
                            <?php
                            if(!empty($_POST['swretail_sync_order'])){
                                if(!empty($_POST['swretail_order_id'])){
                                    $order = wc_get_order( $_POST['swretail_order_id'] );
                                    $swRetail = new SWRetail($swretail_options['licence_key'], 'https://'.$swretail_options['api_url'].'/swcloud/SWWService/', $swretail_options['api_user'], $swretail_options['api_pass']);
                                    $swRetail->postSwOrder($order,99);
                                }else{
                                    _e('To use the sync function Please fill in a order id.', 'swretail');
                                }
                            }
                            ?>
                        </div>
                    </div>
                <?php } ?>
                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'debug'){ 

                    $plugin_data = get_plugin_data( dirname(__FILE__).'/swretail.php' );
                    $plugin_version = $plugin_data['Version'];
                    $filename = 'log-'.str_replace('.','-',$plugin_version);

                    ?>
                    <div id="sw-debug" class="postbox">
                        <h2 class="hndle">Log</h2>
                        <div class="inside">
                            <?php
                            if(file_exists(dirname(__FILE__).'/log/'.$filename.'.txt')){
                                $rawLog = file_get_contents(dirname(__FILE__).'/log/'.$filename.'.txt');
                                $log = explode("\n", $rawLog);
                                $c = 1;
                                foreach ($log as $error){
                                    echo $error.'<br/>';
                                    $c++;
                                    if($c == 20){
                                        break;
                                    }
                                }
                            }else{
                                _e('No errors found', 'swretail');
                            }
                            ?>
                        </div>
                    </div>
                <?php } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'product_bundles'){ ?>
                    <form method="post">
                        <div id="sw-general-settings" class="postbox">
                            <h2 class="hndle"><?php _e('Product Bundles', 'swretail'); ?></h2>
                            <div class="inside">

                                <ol>
                                    <li><?php _e('Create a new product in SW Retail', 'swretail'); ?></li>
                                    <li><?php _e('Just give it a name, so you know it\'s a product bundle', 'swretail'); ?></li>
                                    <li><?php _e('Do not give the product a price and don\'t place the product in webshop', 'swretail'); ?></li>
                                    <li><?php _e('Copy the ID of the product and paste it below', 'swretail'); ?></li>
                                </ol>

                                <table>
                                    <tr>
                                        <td><label for="swretail_addons[wc_product_bundles][sw_prod_id]"><?php _e('SW Retail product ID', 'swretail'); ?> </label></td>
                                        <td>
                                            <input type="number" name="swretail_addons[wc_product_bundles][sw_prod_id]" id="swretail_addons[wc_product_bundles][sw_prod_id]" value="<?php echo (!empty($swretail_addons['wc_product_bundles']['sw_prod_id']) ? $swretail_addons['wc_product_bundles']['sw_prod_id'] : ''); ?>" required />
                                        </td>
                                    </tr>
                                </table>
                                <input type="submit" name="sw_save_addons" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </div>
                        </div>
                    </form>
                <?php } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'effectconnect'){
                    ?>
                    <form method="post">
                        <div id="sw-general-settings" class="postbox">
                            <h2 class="hndle"><?php _e('EffectConnect', 'swretail'); ?></h2>
                            <div class="inside">
                                <table>
                                    <tr>
                                        <td><label for="swretail_addons[effectconnect][order_id]"><?php _e('Order ID in SW Retail', 'swretail'); ?> </label></td>
                                        <td>
                                            <select name="swretail_addons[effectconnect][order_id]" id="swretail_addons[effectconnect][order_id]">
                                                <option value="default"><?php _e('Default', 'swretail'); ?></option>
                                                <option value="effectconnect_order_number" <?php echo ($swretail_addons['effectconnect']['order_id'] == 'effectconnect_order_number' ? 'selected' : ''); ?>>Effectconnect Order Number</option>
                                                <option value="effectconnect_order_number_channel" <?php echo ($swretail_addons['effectconnect']['order_id'] == 'effectconnect_order_number_channel' ? 'selected' : ''); ?>>Effectconnect Order Number Channel</option>
                                            </select>
                                        </td>
                                    </tr>
                                </table>
                                <input type="submit" name="sw_save_addons" class="button button-primary" value="<?php _e('Save', 'swretail'); ?>">
                            </div>
                        </div>
                    </form>
                <?php } ?>

                

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'external_stock'){
                    include('backend/settings/external-stock-settings.php');
                 } ?>

                <?php if(isset($_GET['tab']) && $_GET['tab'] == 'currency_calculator'){
                    include('backend/settings/currency-calculator-settings.php');
                } ?>


            </div>
        </div>
        <?php
    }

    function swretail_menu_item(){
        add_menu_page( "SW-Retail", "SW-Retail" , 'manage_options', 'swretail', 'swretail_auto_load' );
    }
    add_action( 'admin_menu', 'swretail_menu_item' );
}else{
    function swretail_admin_notice_warning() {
        $class = 'notice notice-warning';
        $message = __( 'This plugin requires WooCommerce to function.', 'swretail' );

        printf( '<div class="%1$s"><p>%2$s</p></div>', $class, $message );
    }
    add_action( 'admin_notices', 'swretail_admin_notice_warning' );
}

function m2m_sw_slack($message){
    $swretail_options = get_option('swretail_options');

    if(!isset($swretail_options['swretail_m2m_slack_send_error']) || $swretail_options['swretail_m2m_slack_send_error'] == 'false'){
        return true;
    }

    $plugin_data = get_plugin_data( dirname(__FILE__).'/swretail.php' );
    $plugin_version = $plugin_data['Version'];

    $ch = curl_init("https://slack.com/api/chat.postMessage");
    $data = http_build_query([
        "token" => "xoxp-33048318116-33570150161-267933872437-d7a7b9702c7d1fe41df9c3411b04c2e8",
        "channel" => '#swretailforwoocommerce',
        "text" => '*SW-Retail for WooCommerce* - '.site_url().' ('.$plugin_version.') - '. $message,
        "username" => "Bob Visser",
        "icon_url" => "http://ishetaltijdvoorbobvisser.nl/assets/images/bob-visser.png",
    ]);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    $result = curl_exec($ch);
    curl_close($ch);

    return $result;
}

function swretailUpdateApiRequestCount($request = ''){
    if(empty($request)){return false;}
    if(!get_option('swretail_api_request_amount')){update_option('swretail_api_request_amount',array());}

    $today = current_time('d-m-Y');
    $api_request_amount = maybe_unserialize(get_option('swretail_api_request_amount'));

    if(isset($api_request_amount[date('d-m-Y',strtotime('-1 month'))])){
        unset($api_request_amount[date('d-m-Y',strtotime('-1 month'))]);
    }

    if(!empty($api_request_amount[$today][$request])){
        $api_request_amount[$today][$request] = $api_request_amount[$today][$request] + 1;
    }else{
        $api_request_amount[$today][$request] = 1;
    }

    $today_request_amount = 0;
    foreach($api_request_amount[$today] as $value){
        $today_request_amount = $today_request_amount + $value;
    }

    // Max calls per day = 2400
    // Max calls per day for multiple installations = 1920
    if($today_request_amount > 1920 && empty($api_request_amount[$today]['slack_send'])){
        m2m_sw_slack('API Request Limit Exeeded ('.$today_request_amount.')');
        $api_request_amount[$today]['slack_send'] = 1;
    }

    update_option('swretail_api_request_amount',$api_request_amount);
}

function sw_change_product_price_display($price,$product) {
	if(get_post_meta($product->get_id(),'_swretail_deposit_price',true)){
        $deposit_label = '<span class="swretail-deposit">+ '.wc_price(get_post_meta($product->get_id(),'_swretail_deposit_price',true)).' '.__('deposit','swretail').'</span>';
        $price .= $deposit_label;
    }
    return $price;
}
add_filter( 'woocommerce_get_price_html', 'sw_change_product_price_display', 100, 2 );

add_action( 'woocommerce_before_calculate_totals', 'sw_change_cart_price' );
function sw_change_cart_price( $cart_object ) {
	foreach ( $cart_object->get_cart() as $hash => $value ) {
        if(get_post_meta($value['product_id'],'_swretail_deposit_price',true)){                                  
		    $value['data']->set_price($value['data']->price+get_post_meta($value['product_id'],'_swretail_deposit_price',true));
        }elseif(get_post_meta($value['variation_id'],'_swretail_deposit_price',true)){                                  
		    $value['data']->set_price($value['data']->price+get_post_meta($value['variation_id'],'_swretail_deposit_price',true));
        }
	}
}
function sw_clean_url($string) {
    $string = str_replace(' ', '-', $string); // Replaces all spaces with hyphens.
    $string = preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
 
    return preg_replace('/-+/', '-', $string); // Replaces multiple hyphens with single one.
 }

 function sw_calculate_price($price, $swretail_tax = 21.00, $reversed = false){
    $swretail_addons = get_option('swretail_addons',array());
    // ADDON: Foreign currency calculator

    if(isset($swretail_addons['currency_calculator']['active']) && $swretail_addons['currency_calculator']['active'] == 1){
        if(isset($swretail_addons['currency_calculator']['currency_to']) && isset($swretail_addons['currency_calculator']['currencies'][$swretail_addons['currency_calculator']['currency_to']])){

            if(!$reversed){
                $oldCurrencyPrice = floatVal($price);

                if(isset($swretail_addons['currency_calculator']['recalculate_vat']) && $swretail_addons['currency_calculator']['recalculate_vat'] == 'yes'){

                    // m2m_sw_slack($oldCurrencyPrice);
                    
                    $def_local_tax_rate = reset(WC_Tax::get_rates())['rate'];
                    $price_no_tax = $oldCurrencyPrice / ((100+$swretail_tax)/100);
                    if(get_option('woocommerce_prices_include_tax') == 'yes'){
                        $oldCurrencyPrice = $price_no_tax * ((100+$def_local_tax_rate)/100);
                    }                    
                    // m2m_sw_slack($price_no_tax);
                    // m2m_sw_slack($oldCurrencyPrice);
                    
                }

                // Calculate price in new valuta
                $price = ($oldCurrencyPrice*$swretail_addons['currency_calculator']['currencies'][$swretail_addons['currency_calculator']['currency_to']]);
                
                if(isset($swretail_addons['currency_calculator']['round_prices']) && $swretail_addons['currency_calculator']['round_prices'] == 'up_ending_5_10'){
                    $price = ceil($price/5)*5;
                }

                // m2m_sw_slack($oldCurrencyPrice.' '.$swretail_addons['currency_calculator']['currency_from'].' = '.$price.' '.$swretail_addons['currency_calculator']['currency_to']);
            }else{
                $oldCurrencyPrice = (int)$price;
                $price = ($oldCurrencyPrice/$swretail_addons['currency_calculator']['currencies'][$swretail_addons['currency_calculator']['currency_to']]);

                // m2m_sw_slack($oldCurrencyPrice.' '.$swretail_addons['currency_calculator']['currency_to'].' = '.$price.' '.$swretail_addons['currency_calculator']['currency_from']);
            }
        }
    }

    return $price;
 }